unit ConvertUTF16;

{*
Utf16sjis
Converting the text from UTF-16 to MS932
Copyright 2020 Sogaya

Delphi 2009

*}

{$I+}

interface

uses
  Windows, SysUtils, SurvivorUTF16, FinalEffect;

type AnsiCharArray = array[0..4095] of AnsiChar;
type ByteArray = array[0..223] of Byte;
type CharArray = array[0..4095] of Char;

type
  Converting = class
  private
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    ASCII_C: AnsiChar;
    beginning: Integer;
    BELL_ASCII_KATAKANA: ByteArray;
    buffer: array of Char;
    buffer_length: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: CharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    EMPTY_ANSI: AnsiString;
    file_name: String;
    H_0A_ANSI: AnsiChar;
    H_0A_C: Char;
    H_0D_ANSI: AnsiChar;
    H_0D_C: Char;
    H_FF_C: AnsiChar;
    KANJI_C: AnsiChar;
    KATAKANA_C: AnsiChar;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    NON_KANJI_C: AnsiChar;
    SURROGATE_PAIR_C: AnsiChar;
    table: SurvivorsUTF16;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    y: CharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function ANSI_CHAR_table: AnsiChar;
    function bufferingWriting(comboLength: Integer): Integer;
    function copyingArrayItem(index: Integer): AnsiChar;
    function displayDataText: Boolean;
    function eight_bit_character(b: Byte): AnsiChar;
    function lineCount: Integer;
    function MS932_phrase_size: Integer;
    function newLine: AnsiString;
    function produce_MS932_phrase(s: PChar; s_length: Integer): AnsiCharArray;
    function produceBinaryFile(name: String; no_removing: Boolean): Integer;
    function reading_to_produce_MS932: AnsiCharArray;
    function reading_to_produce_MS932_CRLF(nn: String): Integer;
    function reading_to_produce_MS932_LF(nn: String): Integer;
    function reading_to_produce_MS932_text(nn: String): Integer;
    function renameBinaryFile(name: String; no_removing: Boolean): Integer;
    function skipping_control_to_produce_MS932_CRLF(nn: String): Integer;
    function skipping_control_to_produce_MS932_LF(nn: String): Integer;
    function skipping_control_to_produce_MS932_text(nn: String): Integer;
    function standard_output: Integer;
    function standard_output_of_non_controllers: Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure presetting;
    procedure setting(final_effects: FinalEffects); overload;
    procedure setting(final_effects: FinalEffects; b: Integer); overload;
    procedure setting(final_effects: FinalEffects; fn: String); overload;
  end;

const
  BELL = 7;
  BOM_COUNT = 1;
  BUFFER_ERROR = -1;
  COMBO_ERROR = -3;
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  EXPANSION_FACTOR = 2;
  FILE_ERROR = -2;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_7E = 126; { '~' }
  H_FE = 254;
  H_FF = 255;
  KANJI_SIZE = 2;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  TEMPORARY_BINARY = 'binary.tmp';

implementation

destructor Converting.Destroy;
begin
  buffer := nil;
  combo := nil;
  table.Free;

  inherited Destroy;
end;

function Converting.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function Converting.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;

      for k := 0 to V_SIZE_1 do
      begin
        v[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            V_SIZE := k + 1;
            V_SIZE_1 := V_SIZE - 1;
            break;
          end;

      end;

      BlockWrite(newFile, v, V_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    V_SIZE := SizeOf(v);
    V_SIZE_1 := V_SIZE - 1;
  except
    Halt(1516);
  end;

  Result := exit_code;
end;

function Converting.copyingArrayItem(index: Integer): AnsiChar;
var
  r: AnsiChar;
begin
  if index < 0 then
    begin
      r := #0;
    end
  else
    begin
      if index > V_SIZE_1 then
        begin
          r := #0;
        end
      else
        begin
          r := v[index];
        end;

    end;

  Result := r;
end;

function Converting.displayDataText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    Halt(1517);
  end;

  Result := b;
end;

function Converting.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function Converting.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function Converting.lineCount: Integer;
begin
  Result := count;
end;

function Converting.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function Converting.produce_MS932_phrase(s: PChar; s_length: Integer): AnsiCharArray;
var
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  s_length_1: Integer;
  unformed: Boolean;
  w: Integer;
begin
  beginning := 0;
  carriage_return_appears := False;
  s_length_1 := s_length - 1;
  v := CLEAR_ARRAY;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to s_length_1 do
  begin
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    w := Ord(s[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        v[m] := table.working(w, 1);
        v[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            v[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if s[n] = H_0A_C then
                  begin
                    count := 1;

                    v[m] := H_0A_ANSI;
                    Inc(m);

                    beginning := n + 1;
                    break;
                  end
                else
                  begin
                    if s[n] = H_0D_C then
                      begin
                        v[m] := H_0D_ANSI;
                        Inc(m);

                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        v[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if s[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := 0 to s_length_1 do }

  if beginning > 0 then
    begin
{ Counting lines }
      high_part_of_kanji := H_D840;
      unformed := False;
      for n := beginning to s_length_1 do
      begin
        if m >= V_SIZE_1 then
          begin
            break;
          end;

        w := Ord(s[n]);

        if unformed then
          begin
            low_part_of_kanji := w;
            v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
            v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
            Inc(m, KANJI_SIZE);

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            v[m] := table.working(w, 1);
            v[ m + 1 ] := table.working(w, 2);
            Inc(m, KANJI_SIZE);
          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                v[m] := table.working(w, 1);
                Inc(m);
              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if s[n] = H_0A_C then
                      begin
                        Inc(count);

                        v[m] := H_0A_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        if s[n] = H_0D_C then
                          begin
                            v[m] := H_0D_ANSI;
                            Inc(m);
                          end
                        else
                          begin
                            v[m] := ANSI_CHAR[w];
                            Inc(m);
                          end;

                      end; { if s[n] = H_0A_C then, else }

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end; { for n := beginning to s_length_1 do }

    end;

  V_SIZE := m;
  V_SIZE_1 := V_SIZE - 1;
  Result := v;
end;

function Converting.produceBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, TEMPORARY_BINARY);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;
      BlockRead(testFile, v, V_SIZE, block_has_been_read);
      BlockWrite(newFile, v, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, name);
        Erase(testFile);
      end;

  except
    Halt(1518);
  end;

  Result := sum;
end;

function Converting.reading_to_produce_MS932: AnsiCharArray;
var
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  v := CLEAR_ARRAY;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1519);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        v[m] := table.working(w, 1);
        v[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            v[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    v[m] := H_0A_ANSI;
                    Inc(m);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        v[m] := H_0D_ANSI;
                        Inc(m);

                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        v[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        v[m] := table.working(w, 1);
        v[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            v[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    v[m] := H_0A_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        v[m] := H_0D_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        v[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  V_SIZE := m;
  V_SIZE_1 := V_SIZE - 1;
  Result := v;
end;

function Converting.reading_to_produce_MS932_CRLF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1520);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    combo[m] := H_0D_ANSI;
                    Inc(m);

                    combo[m] := H_0A_ANSI;
                    Inc(m);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    combo[m] := H_0D_ANSI;
                    Inc(m);

                    combo[m] := H_0A_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := bufferingWriting(m);
end;

function Converting.reading_to_produce_MS932_LF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1521);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    combo[m] := H_0A_ANSI;
                    Inc(m);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    combo[m] := H_0A_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := bufferingWriting(m);
end;

function Converting.reading_to_produce_MS932_text(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1522);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    combo[m] := H_0A_ANSI;
                    Inc(m);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    combo[m] := H_0A_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        combo[m] := H_0D_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[w];
                        Inc(m);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := bufferingWriting(m);
end;

function Converting.renameBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, TEMPORARY_BINARY);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;
      BlockRead(testFile, v, V_SIZE, block_has_been_read);
      BlockWrite(newFile, v, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, TEMPORARY_BINARY);
        Erase(testFile);
      end;

  except
    Halt(1523);
  end;

  Result := sum;
end;

function Converting.setBufferSize: Integer;
var
  r: Integer;
begin
  r := buffer_length;

  try
    SetLength(buffer, buffer_length);
  except
    Halt(1524);
  end;

  try
    SetLength(combo, combo_size);
  except
    Halt(1525);
  end;

  Result := r;
end;

function Converting.skipping_control_to_produce_MS932_CRLF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1526);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    combo[m] := H_0D_ANSI;
                    Inc(m);

                    combo[m] := H_0A_ANSI;
                    Inc(m);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            combo[m] := ANSI_CHAR[w];
                            Inc(m);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    combo[m] := H_0D_ANSI;
                    Inc(m);

                    combo[m] := H_0A_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        continue;
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            combo[m] := ANSI_CHAR[w];
                            Inc(m);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := bufferingWriting(m);
end;

function Converting.skipping_control_to_produce_MS932_LF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1527);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    combo[m] := H_0A_ANSI;
                    Inc(m);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            combo[m] := ANSI_CHAR[w];
                            Inc(m);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    combo[m] := H_0A_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        continue;
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            combo[m] := ANSI_CHAR[w];
                            Inc(m);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := bufferingWriting(m);
end;

function Converting.skipping_control_to_produce_MS932_text(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1528);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  m := 0;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    combo[m] := H_0A_ANSI;
                    Inc(m);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            combo[m] := ANSI_CHAR[w];
                            Inc(m);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
        combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
        Inc(m, KANJI_SIZE);

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        combo[m] := table.working(w, 1);
        combo[ m + 1 ] := table.working(w, 2);
        Inc(m, KANJI_SIZE);
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            combo[m] := table.working(w, 1);
            Inc(m);
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    combo[m] := H_0A_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        combo[m] := H_0D_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            combo[m] := ANSI_CHAR[w];
                            Inc(m);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := bufferingWriting(m);
end;

function Converting.MS932_phrase_size: Integer;
begin
  Result := V_SIZE;
end;

function Converting.standard_output: Integer;
var
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1529);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        Write( table.working(w, 1) );
        Write( table.working(w, 2) );
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            Write( table.working(w, 1) );
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    Write(H_0D_ANSI);
                    Write(H_0A_ANSI);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        Write(ANSI_CHAR[w]);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        Write( table.working(w, 1) );
        Write( table.working(w, 2) );
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            Write( table.working(w, 1) );
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    Write(H_0D_ANSI);
                    Write(H_0A_ANSI);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write(ANSI_CHAR[w]);
                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := count;
end;

function Converting.standard_output_of_non_controllers: Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1530);
  end;

{ Finding the first NEW_LINE }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        Write( table.working(w, 1) );
        Write( table.working(w, 2) );
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            Write( table.working(w, 1) );
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    count := 1;

                    Write(H_0D_ANSI);
                    Write(H_0A_ANSI);

                    break;
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        carriage_return_appears := True;
                        NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                        NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                        NEW_LINE := NEW_LINE + H_0A_ANSI;
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            Write(ANSI_CHAR[w]);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then, else }

          end; { if character_type = KATAKANA_C then, else }

      end; { if c = KANJI_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  high_part_of_kanji := H_D840;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if unformed then
      begin
        low_part_of_kanji := w;
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
        Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );

        unformed := False;
        continue;
      end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
    character_type := table.working(w, 0);
    if character_type = KANJI_C then
      begin
        Write( table.working(w, 1) );
        Write( table.working(w, 2) );
      end
    else
      begin
        if character_type = KATAKANA_C then
          begin
            Write( table.working(w, 1) );
          end
        else
          begin
            if character_type = SURROGATE_PAIR_C then
              begin
                high_part_of_kanji := w;
                unformed := True;
                continue;
              end
            else
              begin
                if buffer[n] = H_0A_C then
                  begin
                    Inc(count);

                    Write(H_0D_ANSI);
                    Write(H_0A_ANSI);
                  end
                else
                  begin
                    if buffer[n] = H_0D_C then
                      begin
                        continue;
                      end
                    else
                      begin
                        b := BELL_ASCII_KATAKANA[w];
                        if b = BELL then
                          begin
                            continue; { Removing control characters }
                          end
                        else
                          begin
                            Write(ANSI_CHAR[w]);
                          end;

                      end;

                  end; { if buffer[n] = H_0A_C then, else }

              end; { if character_type = SURROGATE_PAIR_C then }

          end; { if character_type = KATAKANA_C then, else }

      end; { if character_type = KANJI_C then }

  end; { for n := beginning to buffer_length_1 do }

  Result := count;
end;

procedure Converting.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_length < 0 then
    Halt(exit_code);

end;

procedure Converting.presetting;
begin
  BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7;
  BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := 7; BELL_ASCII_KATAKANA[11] := 7;
  BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := 7; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7;
  BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7;
  BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7;
  BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 32; BELL_ASCII_KATAKANA[33] := 33; BELL_ASCII_KATAKANA[34] := 34; BELL_ASCII_KATAKANA[35] := 35;
  BELL_ASCII_KATAKANA[36] := 36; BELL_ASCII_KATAKANA[37] := 37; BELL_ASCII_KATAKANA[38] := 38; BELL_ASCII_KATAKANA[39] := 39; BELL_ASCII_KATAKANA[40] := 40; BELL_ASCII_KATAKANA[41] := 41;
  BELL_ASCII_KATAKANA[42] := 42; BELL_ASCII_KATAKANA[43] := 43; BELL_ASCII_KATAKANA[44] := 44; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 46; BELL_ASCII_KATAKANA[47] := 47;
  BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53;
  BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 58; BELL_ASCII_KATAKANA[59] := 59;
  BELL_ASCII_KATAKANA[60] := 60; BELL_ASCII_KATAKANA[61] := 61; BELL_ASCII_KATAKANA[62] := 62; BELL_ASCII_KATAKANA[63] := 63; BELL_ASCII_KATAKANA[64] := 64; BELL_ASCII_KATAKANA[65] := 65;
  BELL_ASCII_KATAKANA[66] := 66; BELL_ASCII_KATAKANA[67] := 67; BELL_ASCII_KATAKANA[68] := 68; BELL_ASCII_KATAKANA[69] := 69; BELL_ASCII_KATAKANA[70] := 70; BELL_ASCII_KATAKANA[71] := 71;
  BELL_ASCII_KATAKANA[72] := 72; BELL_ASCII_KATAKANA[73] := 73; BELL_ASCII_KATAKANA[74] := 74; BELL_ASCII_KATAKANA[75] := 75; BELL_ASCII_KATAKANA[76] := 76; BELL_ASCII_KATAKANA[77] := 77;
  BELL_ASCII_KATAKANA[78] := 78; BELL_ASCII_KATAKANA[79] := 79; BELL_ASCII_KATAKANA[80] := 80; BELL_ASCII_KATAKANA[81] := 81; BELL_ASCII_KATAKANA[82] := 82; BELL_ASCII_KATAKANA[83] := 83;
  BELL_ASCII_KATAKANA[84] := 84; BELL_ASCII_KATAKANA[85] := 85; BELL_ASCII_KATAKANA[86] := 86; BELL_ASCII_KATAKANA[87] := 87; BELL_ASCII_KATAKANA[88] := 88; BELL_ASCII_KATAKANA[89] := 89;
  BELL_ASCII_KATAKANA[90] := 90; BELL_ASCII_KATAKANA[91] := 91; BELL_ASCII_KATAKANA[92] := 92; BELL_ASCII_KATAKANA[93] := 93; BELL_ASCII_KATAKANA[94] := 94; BELL_ASCII_KATAKANA[95] := 95;
  BELL_ASCII_KATAKANA[96] := 96; BELL_ASCII_KATAKANA[97] := 97; BELL_ASCII_KATAKANA[98] := 98; BELL_ASCII_KATAKANA[99] := 99; BELL_ASCII_KATAKANA[100] := 100; BELL_ASCII_KATAKANA[101] := 101;
  BELL_ASCII_KATAKANA[102] := 102; BELL_ASCII_KATAKANA[103] := 103; BELL_ASCII_KATAKANA[104] := 104; BELL_ASCII_KATAKANA[105] := 105; BELL_ASCII_KATAKANA[106] := 106; BELL_ASCII_KATAKANA[107] := 107;
  BELL_ASCII_KATAKANA[108] := 108; BELL_ASCII_KATAKANA[109] := 109; BELL_ASCII_KATAKANA[110] := 110; BELL_ASCII_KATAKANA[111] := 111; BELL_ASCII_KATAKANA[112] := 112; BELL_ASCII_KATAKANA[113] := 113;
  BELL_ASCII_KATAKANA[114] := 114; BELL_ASCII_KATAKANA[115] := 115; BELL_ASCII_KATAKANA[116] := 116; BELL_ASCII_KATAKANA[117] := 117; BELL_ASCII_KATAKANA[118] := 118; BELL_ASCII_KATAKANA[119] := 119;
  BELL_ASCII_KATAKANA[120] := 120; BELL_ASCII_KATAKANA[121] := 121; BELL_ASCII_KATAKANA[122] := 122; BELL_ASCII_KATAKANA[123] := 123; BELL_ASCII_KATAKANA[124] := 124; BELL_ASCII_KATAKANA[125] := 125;
  BELL_ASCII_KATAKANA[126] := 126; BELL_ASCII_KATAKANA[127] := 7; BELL_ASCII_KATAKANA[128] := 7; BELL_ASCII_KATAKANA[129] := 7; BELL_ASCII_KATAKANA[130] := 7; BELL_ASCII_KATAKANA[131] := 7;
  BELL_ASCII_KATAKANA[132] := 7; BELL_ASCII_KATAKANA[133] := 7; BELL_ASCII_KATAKANA[134] := 7; BELL_ASCII_KATAKANA[135] := 7; BELL_ASCII_KATAKANA[136] := 7; BELL_ASCII_KATAKANA[137] := 7;
  BELL_ASCII_KATAKANA[138] := 7; BELL_ASCII_KATAKANA[139] := 7; BELL_ASCII_KATAKANA[140] := 7; BELL_ASCII_KATAKANA[141] := 7; BELL_ASCII_KATAKANA[142] := 7; BELL_ASCII_KATAKANA[143] := 7;
  BELL_ASCII_KATAKANA[144] := 7; BELL_ASCII_KATAKANA[145] := 7; BELL_ASCII_KATAKANA[146] := 7; BELL_ASCII_KATAKANA[147] := 7; BELL_ASCII_KATAKANA[148] := 7; BELL_ASCII_KATAKANA[149] := 7;
  BELL_ASCII_KATAKANA[150] := 7; BELL_ASCII_KATAKANA[151] := 7; BELL_ASCII_KATAKANA[152] := 7; BELL_ASCII_KATAKANA[153] := 7; BELL_ASCII_KATAKANA[154] := 7; BELL_ASCII_KATAKANA[155] := 7;
  BELL_ASCII_KATAKANA[156] := 7; BELL_ASCII_KATAKANA[157] := 7; BELL_ASCII_KATAKANA[158] := 7; BELL_ASCII_KATAKANA[159] := 7; BELL_ASCII_KATAKANA[160] := 7; BELL_ASCII_KATAKANA[161] := 161;
  BELL_ASCII_KATAKANA[162] := 162; BELL_ASCII_KATAKANA[163] := 163; BELL_ASCII_KATAKANA[164] := 164; BELL_ASCII_KATAKANA[165] := 165; BELL_ASCII_KATAKANA[166] := 166; BELL_ASCII_KATAKANA[167] := 167;
  BELL_ASCII_KATAKANA[168] := 168; BELL_ASCII_KATAKANA[169] := 169; BELL_ASCII_KATAKANA[170] := 170; BELL_ASCII_KATAKANA[171] := 171; BELL_ASCII_KATAKANA[172] := 172; BELL_ASCII_KATAKANA[173] := 173;
  BELL_ASCII_KATAKANA[174] := 174; BELL_ASCII_KATAKANA[175] := 175; BELL_ASCII_KATAKANA[176] := 176; BELL_ASCII_KATAKANA[177] := 177; BELL_ASCII_KATAKANA[178] := 178; BELL_ASCII_KATAKANA[179] := 179;
  BELL_ASCII_KATAKANA[180] := 180; BELL_ASCII_KATAKANA[181] := 181; BELL_ASCII_KATAKANA[182] := 182; BELL_ASCII_KATAKANA[183] := 183; BELL_ASCII_KATAKANA[184] := 184; BELL_ASCII_KATAKANA[185] := 185;
  BELL_ASCII_KATAKANA[186] := 186; BELL_ASCII_KATAKANA[187] := 187; BELL_ASCII_KATAKANA[188] := 188; BELL_ASCII_KATAKANA[189] := 189; BELL_ASCII_KATAKANA[190] := 190; BELL_ASCII_KATAKANA[191] := 191;
  BELL_ASCII_KATAKANA[192] := 192; BELL_ASCII_KATAKANA[193] := 193; BELL_ASCII_KATAKANA[194] := 194; BELL_ASCII_KATAKANA[195] := 195; BELL_ASCII_KATAKANA[196] := 196; BELL_ASCII_KATAKANA[197] := 197;
  BELL_ASCII_KATAKANA[198] := 198; BELL_ASCII_KATAKANA[199] := 199; BELL_ASCII_KATAKANA[200] := 200; BELL_ASCII_KATAKANA[201] := 201; BELL_ASCII_KATAKANA[202] := 202; BELL_ASCII_KATAKANA[203] := 203;
  BELL_ASCII_KATAKANA[204] := 204; BELL_ASCII_KATAKANA[205] := 205; BELL_ASCII_KATAKANA[206] := 206; BELL_ASCII_KATAKANA[207] := 207; BELL_ASCII_KATAKANA[208] := 208; BELL_ASCII_KATAKANA[209] := 209;
  BELL_ASCII_KATAKANA[210] := 210; BELL_ASCII_KATAKANA[211] := 211; BELL_ASCII_KATAKANA[212] := 212; BELL_ASCII_KATAKANA[213] := 213; BELL_ASCII_KATAKANA[214] := 214; BELL_ASCII_KATAKANA[215] := 215;
  BELL_ASCII_KATAKANA[216] := 216; BELL_ASCII_KATAKANA[217] := 217; BELL_ASCII_KATAKANA[218] := 218; BELL_ASCII_KATAKANA[219] := 219; BELL_ASCII_KATAKANA[220] := 220; BELL_ASCII_KATAKANA[221] := 221;
  BELL_ASCII_KATAKANA[222] := 222; BELL_ASCII_KATAKANA[223] := 223;
end;

procedure Converting.setting(final_effects: FinalEffects);
var
  i: Integer;
  m: Integer;
  pascal_string: AnsiString;
begin
{ Setting for converting phrases with no extra memory space }
  buffer_length := 0;
  combo_size := 0;

  beginning := 0;
  file_name := '';
  new_name := '';

  ASCII_C := AnsiChar(#0);
  count := -1;
  EMPTY_ANSI := '';
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SURROGATE_PAIR_C := AnsiChar(#3);

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_ANSI := ANSI_CHAR[H_0A];
  H_0D_ANSI := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

{ Producing a conversion table in the main memory }
  table := SurvivorsUTF16.Create;
  table.setting(final_effects, ANSI_ASCII);

  if final_effects.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

procedure Converting.setting(final_effects: FinalEffects; b: Integer);
var
  i: Integer;
  m: Integer;
  pascal_string: AnsiString;
begin
  buffer_length := b;
  combo_size := EXPANSION_FACTOR * buffer_length;

  beginning := 0;
  file_name := '';
  new_name := '';

  ASCII_C := AnsiChar(#0);
  count := -1;
  EMPTY_ANSI := '';
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SURROGATE_PAIR_C := AnsiChar(#3);

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_ANSI := ANSI_CHAR[H_0A];
  H_0D_ANSI := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SurvivorsUTF16.Create;
  table.setting(final_effects, ANSI_ASCII);

  if final_effects.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

procedure Converting.setting(final_effects: FinalEffects; fn: String);
var
  block_has_been_read: Integer;
  i: Integer;
  m: Integer;
  pascal_string: AnsiString;
  q: array[0..1] of Byte;
  testFile: File;
begin
  buffer_length := -1;
  file_name := fn;

  new_name := '';

  ASCII_C := AnsiChar(#0);
  count := -1;
  EMPTY_ANSI := '';
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SURROGATE_PAIR_C := AnsiChar(#3);

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    if (q[0] = H_FF) and (q[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_length := file_size(file_name) div SIZE_OF_CHAR;
    combo_size := EXPANSION_FACTOR * buffer_length;
  except
    Halt(1531);
  end;

  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_ANSI := ANSI_CHAR[H_0A];
  H_0D_ANSI := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SurvivorsUTF16.Create;
  table.setting(final_effects, ANSI_ASCII);

  if final_effects.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

end.
