unit IntegerQueue;

interface

uses
  SysUtils, ErrorText;

type
  RingBuffer = class
  protected
    NEW_LINE: String;
    queue: array[0..127] of Integer; { queue_size <= 128 - 1 }
    queue_begin: Integer;
    queue_end: Integer;
    x: ErrorTexts;
  public
    constructor Create;
    destructor Destroy; override;
    function display: Integer; overload;
    function display(standard_out: Boolean): String; overload;
    function emptyRing: Boolean;
    function following(index: Integer): Integer;
    function peek: Integer;
    function pop_front: Integer;
    function produce_ring_buffer(size: Integer): Integer;
    function push_back(p: Integer): Integer;
    function queue_size: Integer;
    function sequence_to_integer(s: String): Integer;
    procedure clearRing;
    procedure setting(q: Integer; size: Integer);
  end;

const
  BELL_BYTE = 7;
  C_ERR = 'err';
  EMPTY = '';
  EMPTY_ERROR = 2;
  FULL_HOUSE_ERROR = 3;
  H_0D = 13;
  H_0A = 10;
  NOT_CONVERTED = -1;
  QUEUE_ARRAY_SIZE = 127;
  QUEUE_ERROR_COUNT = 5;
  QUEUE_GROWN = 0;
  QUEUE_SHRUNK = 1;
  SET_LENGTH_ERROR = 4;
  TEXT_FILE = '.txt';

implementation

constructor RingBuffer.Create;
begin
  inherited Create;

end;

destructor RingBuffer.Destroy;
begin

  inherited Destroy;
end;

function RingBuffer.display: Integer;
var
  index: Integer;
  k: Integer;
  s: String;
  size: Integer;
begin
  size := queue_size;
  index := queue_begin;

  for k := 1 to size do
  begin
    s := IntToStr(queue[index]);
    WriteLn(s);
    index := following(index);
  end;

  Result := size;
end;

function RingBuffer.display(standard_out: Boolean): String;
var
  combo: String;
  index: Integer;
  k: Integer;
  s: String;
  size: Integer;
begin
  combo := EMPTY;
  size := queue_size;
  index := queue_begin;

  for k := 1 to size do
  begin
    s := IntToStr(queue[index]);
    combo := combo + s;
    combo := combo + NEW_LINE;
    index := following(index);
  end;

  if standard_out then
    WriteLn(combo);

  Result := combo;
end;

function RingBuffer.emptyRing: Boolean;
var
  b: Boolean;
begin
  b := False;
  if queue_begin = queue_end then
    b := True;

  Result := b;
end;

function RingBuffer.following(index: Integer): Integer;
var
  k: Integer;
begin
  k := (index + 1) mod QUEUE_ARRAY_SIZE; 
  Result := k; 
end;

function RingBuffer.peek: Integer;
begin
  Result := queue[queue_begin];
end;

function RingBuffer.pop_front: Integer;
var
  r: Integer;
begin
  if emptyRing then
    begin
      r := EMPTY_ERROR;
    end
  else
    begin
      queue_begin := following(queue_begin);
      r := QUEUE_SHRUNK;
    end;

  Result := r;
end;

function RingBuffer.produce_ring_buffer(size: Integer): Integer;
var
  k: Integer;
  r: Integer;
begin
  clearRing;

  for k := 1 to size do
  begin
    push_back(BELL_BYTE);
  end;

  r := queue_size;
  Result := r;
end;

function RingBuffer.push_back(p: Integer): Integer;
var
  index: Integer;
  r: Integer;
begin
  index := following(queue_end);
  if index = queue_begin then
    begin
      r := FULL_HOUSE_ERROR;
    end
  else
    begin
      queue[queue_end] := p;
      queue_end := index;
      r := QUEUE_GROWN;
    end;

  Result := r;
end;

function RingBuffer.queue_size: Integer;
var
  size: Integer;
begin
  if queue_begin < queue_end then
    begin
      size := queue_end - queue_begin;
    end
  else
    begin
      if queue_begin = queue_end then
        begin
          size := 0;
        end
      else
        begin
          size := queue_end + QUEUE_ARRAY_SIZE - queue_begin;
        end;

    end;

  Result := size;
end;

function RingBuffer.sequence_to_integer(s: String): Integer;
var
  k: Integer;
  messages: String;
begin
  k := NOT_CONVERTED;

  try
    k := StrToInt(s);
  except
{ EConvertError }
    x := ErrorTexts.Create;
    messages := 'sequence_to_integer() error-415: ' + s;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
  end;

  Result := k;
end;

procedure RingBuffer.clearRing;
var
  k: Integer;
  repetition: Integer;
begin
  repetition := queue_size;

  for k := 1 to repetition do
  begin
    pop_front;
  end;

end;

procedure RingBuffer.setting(q: Integer; size: Integer);
var
  k: Integer;
begin
  NEW_LINE := EMPTY + Chr(H_0D);
  NEW_LINE := NEW_LINE + Chr(H_0A);

  for k := 1 to size do
  begin
    push_back(q);
  end;

end;

end.
