unit StandardIO;

{$I+}

interface

uses
  Windows, SysUtils, ErrorText, SparseEUC, FinalEffect, SearchBinary;

type ByteArray = array[0..223] of Byte;
type AnsiCharArray = array[0..4095] of AnsiChar;

type
  StandardInput = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ANSI_CHAR: array[0..255] of AnsiChar;
    beginning: Integer;
    BELL_ASCII_KATAKANA: ByteArray;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    EMPTY_ANSI: AnsiString;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    table: SparsePopulationEUC;
    x: ErrorTexts;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function ANSI_CHAR_table: AnsiChar;
    function bufferingWriting(comboLength: Integer): Integer;
    function displayDataText: Boolean;
    function eight_bit_character(b: Byte): AnsiChar;
    function lineCount: Integer;
    function newLine: AnsiString;
    function produceTemporaryTextFile: Integer;
    function standard_input: Integer;
    function standard_input_of_non_controllers: Integer;
    procedure presetting;
    procedure setting(final_effects: FinalEffects); overload;
    procedure setting(final_effects: FinalEffects; b: Integer); overload;
    procedure setting(search_table: SearchTable); overload;
  end;

const
  BELL = 7;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  COMBO_ERROR = -3;
  COMMA_BYTE = 44; { 2C, ',' }
  COMMAND_NAME_FILE = 'sjisEUC.txt';
  DATA_TEXT = 'data.txt';
  EXPANSION_FACTOR = 2;
  FILE_ERROR = -2;
  H_00FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_5C = 92; { '\' }
  H_80 = 128;
  H_81 = 129;
  H_FC = 252;
  H_FE = 254;
  H_FF = 255;
  KANA_HANDAKUTEN = 223; { DFh }
  KANA_KUTEN = 161; { A1h }
  KANJI_SIZE = 2;
  SIZE_OF_ANSICHAR = 1;
  TEMPORARY_TEXT = 'text.tmp';
  TEXT_FILE = '.txt';

implementation

destructor StandardInput.Destroy;
begin
  buffer := nil;
  combo := nil;
  table.Free;

  inherited Destroy;
end;

function StandardInput.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function StandardInput.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;

      for k := 0 to A_SIZE_1 do
      begin
        a[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            A_SIZE := k + 1;
            break;
          end;

      end;

      BlockWrite(newFile, a, A_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    A_SIZE := SizeOf(a);
  except
    s := 'Rewrite() error-474: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function StandardInput.displayDataText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-475: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function StandardInput.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function StandardInput.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function StandardInput.lineCount: Integer;
begin
  Result := count;
end;

function StandardInput.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function StandardInput.produceTemporaryTextFile: Integer;
const
  NEW_LINE_SIZE = 2; { CR, LF }
var
  newFile: TextFile;
  s: String;
  sum: Integer;
begin
  sum := 0;

  try
    AssignFile(newFile, TEMPORARY_TEXT);
    Rewrite(newFile);

    while not Eof do
    begin
      ReadLn(s);
      WriteLn(newFile, s);
      sum := sum + Length(s);
      sum := sum + NEW_LINE_SIZE;
    end;

    CloseFile(newFile);
  except
    s := 'Reset() error-476: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function StandardInput.standard_input: Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  m: Integer;
  kanji: Boolean;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, TEMPORARY_TEXT);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n:= 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-477: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := ANSI_CHAR[COMMA_BYTE];
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0A_C;
            Inc(m);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                Inc(m);

                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        Inc(m);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    Inc(m);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := ANSI_CHAR[COMMA_BYTE];
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0A_C;
            Inc(m);
          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                Inc(m);
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        Inc(m);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    Inc(m);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function StandardInput.standard_input_of_non_controllers: Integer;
var
  b: Integer;
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  kanji: Boolean;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, TEMPORARY_TEXT);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n:= 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-478: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := ANSI_CHAR[COMMA_BYTE];
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0A_C;
            Inc(m);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                Inc(m);

                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        Inc(m);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    b := BELL_ASCII_KATAKANA[k];
                    if b = BELL then
                      begin
                        continue; { Removing control characters }
                      end
                    else
                      begin
                        combo[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := ANSI_CHAR[COMMA_BYTE];
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0A_C;
            Inc(m);

          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                Inc(m);
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        Inc(m);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    b := BELL_ASCII_KATAKANA[k];
                    if b = BELL then
                      begin
                        continue; { Removing control characters }
                      end
                    else
                      begin
                        combo[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function StandardInput.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-479: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-480: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

procedure StandardInput.presetting;
begin
  BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7; 
  BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := 7; BELL_ASCII_KATAKANA[11] := 7; 
  BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := 7; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7; 
  BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7; 
  BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7; 
  BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 32; BELL_ASCII_KATAKANA[33] := 33; BELL_ASCII_KATAKANA[34] := 34; BELL_ASCII_KATAKANA[35] := 35; 
  BELL_ASCII_KATAKANA[36] := 36; BELL_ASCII_KATAKANA[37] := 37; BELL_ASCII_KATAKANA[38] := 38; BELL_ASCII_KATAKANA[39] := 39; BELL_ASCII_KATAKANA[40] := 40; BELL_ASCII_KATAKANA[41] := 41; 
  BELL_ASCII_KATAKANA[42] := 42; BELL_ASCII_KATAKANA[43] := 43; BELL_ASCII_KATAKANA[44] := 44; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 46; BELL_ASCII_KATAKANA[47] := 47; 
  BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53; 
  BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 58; BELL_ASCII_KATAKANA[59] := 59; 
  BELL_ASCII_KATAKANA[60] := 60; BELL_ASCII_KATAKANA[61] := 61; BELL_ASCII_KATAKANA[62] := 62; BELL_ASCII_KATAKANA[63] := 63; BELL_ASCII_KATAKANA[64] := 64; BELL_ASCII_KATAKANA[65] := 65; 
  BELL_ASCII_KATAKANA[66] := 66; BELL_ASCII_KATAKANA[67] := 67; BELL_ASCII_KATAKANA[68] := 68; BELL_ASCII_KATAKANA[69] := 69; BELL_ASCII_KATAKANA[70] := 70; BELL_ASCII_KATAKANA[71] := 71; 
  BELL_ASCII_KATAKANA[72] := 72; BELL_ASCII_KATAKANA[73] := 73; BELL_ASCII_KATAKANA[74] := 74; BELL_ASCII_KATAKANA[75] := 75; BELL_ASCII_KATAKANA[76] := 76; BELL_ASCII_KATAKANA[77] := 77; 
  BELL_ASCII_KATAKANA[78] := 78; BELL_ASCII_KATAKANA[79] := 79; BELL_ASCII_KATAKANA[80] := 80; BELL_ASCII_KATAKANA[81] := 81; BELL_ASCII_KATAKANA[82] := 82; BELL_ASCII_KATAKANA[83] := 83; 
  BELL_ASCII_KATAKANA[84] := 84; BELL_ASCII_KATAKANA[85] := 85; BELL_ASCII_KATAKANA[86] := 86; BELL_ASCII_KATAKANA[87] := 87; BELL_ASCII_KATAKANA[88] := 88; BELL_ASCII_KATAKANA[89] := 89; 
  BELL_ASCII_KATAKANA[90] := 90; BELL_ASCII_KATAKANA[91] := 91; BELL_ASCII_KATAKANA[92] := 92; BELL_ASCII_KATAKANA[93] := 93; BELL_ASCII_KATAKANA[94] := 94; BELL_ASCII_KATAKANA[95] := 95; 
  BELL_ASCII_KATAKANA[96] := 96; BELL_ASCII_KATAKANA[97] := 97; BELL_ASCII_KATAKANA[98] := 98; BELL_ASCII_KATAKANA[99] := 99; BELL_ASCII_KATAKANA[100] := 100; BELL_ASCII_KATAKANA[101] := 101; 
  BELL_ASCII_KATAKANA[102] := 102; BELL_ASCII_KATAKANA[103] := 103; BELL_ASCII_KATAKANA[104] := 104; BELL_ASCII_KATAKANA[105] := 105; BELL_ASCII_KATAKANA[106] := 106; BELL_ASCII_KATAKANA[107] := 107; 
  BELL_ASCII_KATAKANA[108] := 108; BELL_ASCII_KATAKANA[109] := 109; BELL_ASCII_KATAKANA[110] := 110; BELL_ASCII_KATAKANA[111] := 111; BELL_ASCII_KATAKANA[112] := 112; BELL_ASCII_KATAKANA[113] := 113; 
  BELL_ASCII_KATAKANA[114] := 114; BELL_ASCII_KATAKANA[115] := 115; BELL_ASCII_KATAKANA[116] := 116; BELL_ASCII_KATAKANA[117] := 117; BELL_ASCII_KATAKANA[118] := 118; BELL_ASCII_KATAKANA[119] := 119; 
  BELL_ASCII_KATAKANA[120] := 120; BELL_ASCII_KATAKANA[121] := 121; BELL_ASCII_KATAKANA[122] := 122; BELL_ASCII_KATAKANA[123] := 123; BELL_ASCII_KATAKANA[124] := 124; BELL_ASCII_KATAKANA[125] := 125; 
  BELL_ASCII_KATAKANA[126] := 126; BELL_ASCII_KATAKANA[127] := 7; BELL_ASCII_KATAKANA[128] := 7; BELL_ASCII_KATAKANA[129] := 7; BELL_ASCII_KATAKANA[130] := 7; BELL_ASCII_KATAKANA[131] := 7; 
  BELL_ASCII_KATAKANA[132] := 7; BELL_ASCII_KATAKANA[133] := 7; BELL_ASCII_KATAKANA[134] := 7; BELL_ASCII_KATAKANA[135] := 7; BELL_ASCII_KATAKANA[136] := 7; BELL_ASCII_KATAKANA[137] := 7; 
  BELL_ASCII_KATAKANA[138] := 7; BELL_ASCII_KATAKANA[139] := 7; BELL_ASCII_KATAKANA[140] := 7; BELL_ASCII_KATAKANA[141] := 7; BELL_ASCII_KATAKANA[142] := 7; BELL_ASCII_KATAKANA[143] := 7; 
  BELL_ASCII_KATAKANA[144] := 7; BELL_ASCII_KATAKANA[145] := 7; BELL_ASCII_KATAKANA[146] := 7; BELL_ASCII_KATAKANA[147] := 7; BELL_ASCII_KATAKANA[148] := 7; BELL_ASCII_KATAKANA[149] := 7; 
  BELL_ASCII_KATAKANA[150] := 7; BELL_ASCII_KATAKANA[151] := 7; BELL_ASCII_KATAKANA[152] := 7; BELL_ASCII_KATAKANA[153] := 7; BELL_ASCII_KATAKANA[154] := 7; BELL_ASCII_KATAKANA[155] := 7; 
  BELL_ASCII_KATAKANA[156] := 7; BELL_ASCII_KATAKANA[157] := 7; BELL_ASCII_KATAKANA[158] := 7; BELL_ASCII_KATAKANA[159] := 7; BELL_ASCII_KATAKANA[160] := 7; BELL_ASCII_KATAKANA[161] := 161; 
  BELL_ASCII_KATAKANA[162] := 162; BELL_ASCII_KATAKANA[163] := 163; BELL_ASCII_KATAKANA[164] := 164; BELL_ASCII_KATAKANA[165] := 165; BELL_ASCII_KATAKANA[166] := 166; BELL_ASCII_KATAKANA[167] := 167; 
  BELL_ASCII_KATAKANA[168] := 168; BELL_ASCII_KATAKANA[169] := 169; BELL_ASCII_KATAKANA[170] := 170; BELL_ASCII_KATAKANA[171] := 171; BELL_ASCII_KATAKANA[172] := 172; BELL_ASCII_KATAKANA[173] := 173; 
  BELL_ASCII_KATAKANA[174] := 174; BELL_ASCII_KATAKANA[175] := 175; BELL_ASCII_KATAKANA[176] := 176; BELL_ASCII_KATAKANA[177] := 177; BELL_ASCII_KATAKANA[178] := 178; BELL_ASCII_KATAKANA[179] := 179; 
  BELL_ASCII_KATAKANA[180] := 180; BELL_ASCII_KATAKANA[181] := 181; BELL_ASCII_KATAKANA[182] := 182; BELL_ASCII_KATAKANA[183] := 183; BELL_ASCII_KATAKANA[184] := 184; BELL_ASCII_KATAKANA[185] := 185; 
  BELL_ASCII_KATAKANA[186] := 186; BELL_ASCII_KATAKANA[187] := 187; BELL_ASCII_KATAKANA[188] := 188; BELL_ASCII_KATAKANA[189] := 189; BELL_ASCII_KATAKANA[190] := 190; BELL_ASCII_KATAKANA[191] := 191; 
  BELL_ASCII_KATAKANA[192] := 192; BELL_ASCII_KATAKANA[193] := 193; BELL_ASCII_KATAKANA[194] := 194; BELL_ASCII_KATAKANA[195] := 195; BELL_ASCII_KATAKANA[196] := 196; BELL_ASCII_KATAKANA[197] := 197; 
  BELL_ASCII_KATAKANA[198] := 198; BELL_ASCII_KATAKANA[199] := 199; BELL_ASCII_KATAKANA[200] := 200; BELL_ASCII_KATAKANA[201] := 201; BELL_ASCII_KATAKANA[202] := 202; BELL_ASCII_KATAKANA[203] := 203; 
  BELL_ASCII_KATAKANA[204] := 204; BELL_ASCII_KATAKANA[205] := 205; BELL_ASCII_KATAKANA[206] := 206; BELL_ASCII_KATAKANA[207] := 207; BELL_ASCII_KATAKANA[208] := 208; BELL_ASCII_KATAKANA[209] := 209; 
  BELL_ASCII_KATAKANA[210] := 210; BELL_ASCII_KATAKANA[211] := 211; BELL_ASCII_KATAKANA[212] := 212; BELL_ASCII_KATAKANA[213] := 213; BELL_ASCII_KATAKANA[214] := 214; BELL_ASCII_KATAKANA[215] := 215; 
  BELL_ASCII_KATAKANA[216] := 216; BELL_ASCII_KATAKANA[217] := 217; BELL_ASCII_KATAKANA[218] := 218; BELL_ASCII_KATAKANA[219] := 219; BELL_ASCII_KATAKANA[220] := 220; BELL_ASCII_KATAKANA[221] := 221; 
  BELL_ASCII_KATAKANA[222] := 222; BELL_ASCII_KATAKANA[223] := 223;
end;

procedure StandardInput.setting(final_effects: FinalEffects; b: Integer);
var
  i: Integer;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  buffer_size := b;
  combo_size := EXPANSION_FACTOR * buffer_size;

  count := -1;
  EMPTY_ANSI := '';

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];
  H_FE_C := ANSI_CHAR[H_FE];

  NEW_LINE := EMPTY_ANSI + ANSI_CHAR[H_0A];
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SparsePopulationEUC.Create;
  table.setting(final_effects);
end;

procedure StandardInput.setting(final_effects: FinalEffects);
var
  i: Integer;
  s: String;
  testFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  buffer_size := -1;
  new_name := final_effects.newName;

  try
    AssignFile(testFile, TEMPORARY_TEXT);
    Reset(testFile, SIZE_OF_ANSICHAR);
    CloseFile(testFile);

    buffer_size := file_size(TEMPORARY_TEXT);
    combo_size := EXPANSION_FACTOR * buffer_size;
  except
    s := 'Reset() error-481: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  count := -1;
  EMPTY_ANSI := '';

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];
  H_FE_C := ANSI_CHAR[H_FE];

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SparsePopulationEUC.Create;
  table.setting(final_effects);
end;

procedure StandardInput.setting(search_table: SearchTable);
var
  i: Integer;
  s: String;
  testFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  buffer_size := -1;
  new_name := COMMAND_NAME_FILE;

  try
    AssignFile(testFile, TEMPORARY_TEXT);
    Reset(testFile, SIZE_OF_ANSICHAR);
    CloseFile(testFile);

    buffer_size := file_size(TEMPORARY_TEXT);
    combo_size := EXPANSION_FACTOR * buffer_size;
  except
    s := 'Reset() error-482: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  count := -1;
  EMPTY_ANSI := '';

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];
  H_FE_C := ANSI_CHAR[H_FE];

  NEW_LINE := EMPTY_ANSI + ANSI_CHAR[H_0A];
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SparsePopulationEUC.Create;
  table.setting(search_table);
end;

end.
