unit SelectField;

{$I+}

{*
Software name: EUC-JP Unarfij
Selecting a field from the UTF-16 file to produce an EUC-JP text
Copyright 2020 Sogaya

Delphi 2009

*}

interface

uses
  Windows, SysUtils, ErrorText, PastoralUTF16, FinalEffect;

type AnsiCharArray = array[0..4095] of AnsiChar;
type ByteArray = array[0..223] of Byte;
type CharArray = array[0..4095] of Char;

type
  Selecting = class
  private
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    ASCII_C: AnsiChar;
    beginning: Integer;
    BELL_ASCII_KATAKANA: ByteArray;
    buffer: array of Char;
    buffer_length: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: CharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    EMPTY_ANSI: AnsiString;
    f: FinalEffects;
    file_name: String;
    H_0A_ANSI: AnsiChar;
    H_0D_ANSI: AnsiChar;
    H_FF_C: AnsiChar;
    KANJI_C: AnsiChar;
    KATAKANA_C: AnsiChar;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    NON_KANJI_C: AnsiChar;
    quote_byte: Byte;
    remove_separators: Boolean;
    field_to_be_selected: Integer;
    separator_byte: Byte;
    SURROGATE_PAIR_C: AnsiChar;
    table: PastoralSceneUTF16;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    x: ErrorTexts;
    y: CharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function ANSI_CHAR_table: AnsiChar;
    function bufferingWriting(comboLength: Integer): Integer;
    function copyingArrayItem(index: Integer): AnsiChar;
    function displayDataText: Boolean;
    function eight_bit_character(b: Byte): AnsiChar;
    function lineCount: Integer;
    function EUC_phrase_size: Integer;
    function newLine: AnsiString;
    function reading_to_produce_EUC_CRLF(nn: String): Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure presetting(final_effects: FinalEffects);
    procedure setting(final_effects: FinalEffects; b: Integer); overload;
    procedure setting(final_effects: FinalEffects; fn: String); overload;
  end;

const
  BELL = 7;
  BOM_COUNT = 1;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  COMBO_ERROR = -3;
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  EXPANSION_FACTOR = 2;
  FIRST_FIELD = 1;
  FILE_ERROR = -2;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_22 = 34; { 22h, '"' }
  H_2C = 44; { 2Ch, ',' }
  H_3E = 62; { '>' }
  H_7E = 126; { '~' }
  H_FE = 254;
  H_FF = 255;
  KANJI_SIZE = 2;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  TEXT_FILE = '.txt';

implementation

destructor Selecting.Destroy;
begin
  buffer := nil;
  combo := nil;
  table.Free;

  inherited Destroy;
end;

function Selecting.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function Selecting.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;

      for k := 0 to V_SIZE_1 do
      begin
        v[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            V_SIZE := k + 1;
            V_SIZE_1 := V_SIZE - 1;
            break;
          end;

      end;

      BlockWrite(newFile, v, V_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    V_SIZE := SizeOf(v);
    V_SIZE_1 := V_SIZE - 1;
  except
    s := 'Rewrite() error-357: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function Selecting.copyingArrayItem(index: Integer): AnsiChar;
var
  r: AnsiChar;
begin
  if index < 0 then
    begin
      r := #0;
    end
  else
    begin
      if index > V_SIZE_1 then
        begin
          r := #0;
        end
      else
        begin
          r := v[index];
        end;

    end;

  Result := r;
end;

function Selecting.displayDataText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-358: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function Selecting.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function Selecting.file_size(name: String): Integer;
var
  t: Integer;
  size: Integer;
begin
  t := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(t, 0, 2); { EOF }
  FileClose(t);
  Result := size;
end;

function Selecting.lineCount: Integer;
begin
  Result := count;
end;

function Selecting.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function Selecting.reading_to_produce_EUC_CRLF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n := 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-359: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[b];
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := ANSI_CHAR[w];
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := ANSI_CHAR[w];
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[b];
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := ANSI_CHAR[w];
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := ANSI_CHAR[w];
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[b];
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := ANSI_CHAR[w];
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := ANSI_CHAR[w];
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := ANSI_CHAR[b];
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := ANSI_CHAR[w];
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := ANSI_CHAR[w];
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_length;

  try
    SetLength(buffer, buffer_length);
  except
    s := 'SetLength() error-360: ' + IntToStr(buffer_length);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-361: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

function Selecting.EUC_phrase_size: Integer;
begin
  Result := V_SIZE;
end;

procedure Selecting.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_length < 0 then
    Halt(exit_code);

end;

procedure Selecting.presetting(final_effects: FinalEffects);
var
  k: Integer;
begin
  f := final_effects;

  BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7;
  BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := 7; BELL_ASCII_KATAKANA[11] := 7;
  BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := 7; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7;
  BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7;
  BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7;
  BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 32; BELL_ASCII_KATAKANA[33] := 33; BELL_ASCII_KATAKANA[34] := 34; BELL_ASCII_KATAKANA[35] := 35;
  BELL_ASCII_KATAKANA[36] := 36; BELL_ASCII_KATAKANA[37] := 37; BELL_ASCII_KATAKANA[38] := 38; BELL_ASCII_KATAKANA[39] := 39; BELL_ASCII_KATAKANA[40] := 40; BELL_ASCII_KATAKANA[41] := 41;
  BELL_ASCII_KATAKANA[42] := 42; BELL_ASCII_KATAKANA[43] := 43; BELL_ASCII_KATAKANA[44] := 44; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 46; BELL_ASCII_KATAKANA[47] := 47;
  BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53;
  BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 58; BELL_ASCII_KATAKANA[59] := 59;
  BELL_ASCII_KATAKANA[60] := 60; BELL_ASCII_KATAKANA[61] := 61; BELL_ASCII_KATAKANA[62] := 62; BELL_ASCII_KATAKANA[63] := 63; BELL_ASCII_KATAKANA[64] := 64; BELL_ASCII_KATAKANA[65] := 65;
  BELL_ASCII_KATAKANA[66] := 66; BELL_ASCII_KATAKANA[67] := 67; BELL_ASCII_KATAKANA[68] := 68; BELL_ASCII_KATAKANA[69] := 69; BELL_ASCII_KATAKANA[70] := 70; BELL_ASCII_KATAKANA[71] := 71;
  BELL_ASCII_KATAKANA[72] := 72; BELL_ASCII_KATAKANA[73] := 73; BELL_ASCII_KATAKANA[74] := 74; BELL_ASCII_KATAKANA[75] := 75; BELL_ASCII_KATAKANA[76] := 76; BELL_ASCII_KATAKANA[77] := 77;
  BELL_ASCII_KATAKANA[78] := 78; BELL_ASCII_KATAKANA[79] := 79; BELL_ASCII_KATAKANA[80] := 80; BELL_ASCII_KATAKANA[81] := 81; BELL_ASCII_KATAKANA[82] := 82; BELL_ASCII_KATAKANA[83] := 83;
  BELL_ASCII_KATAKANA[84] := 84; BELL_ASCII_KATAKANA[85] := 85; BELL_ASCII_KATAKANA[86] := 86; BELL_ASCII_KATAKANA[87] := 87; BELL_ASCII_KATAKANA[88] := 88; BELL_ASCII_KATAKANA[89] := 89;
  BELL_ASCII_KATAKANA[90] := 90; BELL_ASCII_KATAKANA[91] := 91; BELL_ASCII_KATAKANA[92] := 92; BELL_ASCII_KATAKANA[93] := 93; BELL_ASCII_KATAKANA[94] := 94; BELL_ASCII_KATAKANA[95] := 95;
  BELL_ASCII_KATAKANA[96] := 96; BELL_ASCII_KATAKANA[97] := 97; BELL_ASCII_KATAKANA[98] := 98; BELL_ASCII_KATAKANA[99] := 99; BELL_ASCII_KATAKANA[100] := 100; BELL_ASCII_KATAKANA[101] := 101;
  BELL_ASCII_KATAKANA[102] := 102; BELL_ASCII_KATAKANA[103] := 103; BELL_ASCII_KATAKANA[104] := 104; BELL_ASCII_KATAKANA[105] := 105; BELL_ASCII_KATAKANA[106] := 106; BELL_ASCII_KATAKANA[107] := 107;
  BELL_ASCII_KATAKANA[108] := 108; BELL_ASCII_KATAKANA[109] := 109; BELL_ASCII_KATAKANA[110] := 110; BELL_ASCII_KATAKANA[111] := 111; BELL_ASCII_KATAKANA[112] := 112; BELL_ASCII_KATAKANA[113] := 113;
  BELL_ASCII_KATAKANA[114] := 114; BELL_ASCII_KATAKANA[115] := 115; BELL_ASCII_KATAKANA[116] := 116; BELL_ASCII_KATAKANA[117] := 117; BELL_ASCII_KATAKANA[118] := 118; BELL_ASCII_KATAKANA[119] := 119;
  BELL_ASCII_KATAKANA[120] := 120; BELL_ASCII_KATAKANA[121] := 121; BELL_ASCII_KATAKANA[122] := 122; BELL_ASCII_KATAKANA[123] := 123; BELL_ASCII_KATAKANA[124] := 124; BELL_ASCII_KATAKANA[125] := 125;
  BELL_ASCII_KATAKANA[126] := 126; BELL_ASCII_KATAKANA[127] := 7; BELL_ASCII_KATAKANA[128] := 7; BELL_ASCII_KATAKANA[129] := 7; BELL_ASCII_KATAKANA[130] := 7; BELL_ASCII_KATAKANA[131] := 7;
  BELL_ASCII_KATAKANA[132] := 7; BELL_ASCII_KATAKANA[133] := 7; BELL_ASCII_KATAKANA[134] := 7; BELL_ASCII_KATAKANA[135] := 7; BELL_ASCII_KATAKANA[136] := 7; BELL_ASCII_KATAKANA[137] := 7;
  BELL_ASCII_KATAKANA[138] := 7; BELL_ASCII_KATAKANA[139] := 7; BELL_ASCII_KATAKANA[140] := 7; BELL_ASCII_KATAKANA[141] := 7; BELL_ASCII_KATAKANA[142] := 7; BELL_ASCII_KATAKANA[143] := 7;
  BELL_ASCII_KATAKANA[144] := 7; BELL_ASCII_KATAKANA[145] := 7; BELL_ASCII_KATAKANA[146] := 7; BELL_ASCII_KATAKANA[147] := 7; BELL_ASCII_KATAKANA[148] := 7; BELL_ASCII_KATAKANA[149] := 7;
  BELL_ASCII_KATAKANA[150] := 7; BELL_ASCII_KATAKANA[151] := 7; BELL_ASCII_KATAKANA[152] := 7; BELL_ASCII_KATAKANA[153] := 7; BELL_ASCII_KATAKANA[154] := 7; BELL_ASCII_KATAKANA[155] := 7;
  BELL_ASCII_KATAKANA[156] := 7; BELL_ASCII_KATAKANA[157] := 7; BELL_ASCII_KATAKANA[158] := 7; BELL_ASCII_KATAKANA[159] := 7; BELL_ASCII_KATAKANA[160] := 7; 

  for k := 161 to 223 do
  begin
    BELL_ASCII_KATAKANA[k] := k;
  end;

end;

procedure Selecting.setting(final_effects: FinalEffects; b: Integer);
var
  i: Integer;
  m: Integer;
  pascal_string: AnsiString;
begin
  buffer_length := b;
  combo_size := EXPANSION_FACTOR * buffer_length;
  f := final_effects;

  beginning := 0;
  file_name := '';
  field_to_be_selected := f.fieldSelected;
  new_name := '';
  quote_byte := H_22;
  remove_separators := False;
  separator_byte := H_2C;

  ASCII_C := AnsiChar(#0);
  count := -1;
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SURROGATE_PAIR_C := AnsiChar(#3);

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_ANSI := ANSI_CHAR[H_0A];
  H_0D_ANSI := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := PastoralSceneUTF16.Create;
  table.setting(f, ANSI_ASCII);

  if f.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

procedure Selecting.setting(final_effects: FinalEffects; fn: String);
var
  block_has_been_read: Integer;
  i: Integer;
  m: Integer;
  pascal_string: AnsiString;
  q: array[0..1] of Byte;
  s: String;
  testFile: File;
begin
  buffer_length := -1;
  f := final_effects;
  file_name := fn;
  field_to_be_selected := f.fieldSelected;
  new_name := '';
  quote_byte := H_22;
  remove_separators := False;
  separator_byte := H_2C;

  ASCII_C := AnsiChar(#0);
  count := -1;
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SURROGATE_PAIR_C := AnsiChar(#3);

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    if (q[0] = H_FF) and (q[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_length := file_size(file_name) div SIZE_OF_CHAR;
    combo_size := EXPANSION_FACTOR * buffer_length;
  except
    s := 'Reset() error-362: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_ANSI := ANSI_CHAR[H_0A];
  H_0D_ANSI := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := PastoralSceneUTF16.Create;
  table.setting(f, ANSI_ASCII);

  if f.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

end.
